import httpx

from mack_SDK.resources.movie import Movie
from mack_SDK.resources.quote import Quote
from mack_SDK.tools.settings import Settings


class Client(object):
    """Create LOTR API Client.

    This class is used to create a client for the LOTR API.
    API docs - https://the-one-api.dev/documentation.
    `BEARER_TOKEN` is required for the API to work ([get token](https://the-one-api.dev/sign-up)),
    it may be passed as an argument or set as an environment variable.
    Currently not supported: Filtering, Sorting, Pagination (needs to be implemented by the client
    user).

    Examples:
        ```python
        # Initialize a client
        import mack_SDK.client
        client = mack_SDK.client.Client(bearer_token="some_token")

        # Work with `/movies` resources:
        movie_client = client.create_resource_class("movie")
        movies = movie_client.get_all_movies(page=2)
        movie = movie_client.get_movie("5cd95395de30eff6ebccde5b")
        quotes = movie_client.get_quotes("5cd95395de30eff6ebccde5b", limit=10)

        # Work with `/quotes` resources:
        movie_client = client.create_resource_class("movie")
        quote_client = client.create_resource_class("quote")
        quotes = quote_client.get_all_quotes(limit=50, page=2)
        quote = quote_client.get_quote("5cd96e05de30eff6ebcce7ea")
    ```
    """

    http_client: httpx.Client

    def __init__(self, base_url: str = "https://the-one-api.dev/v2", bearer_token: str = None):
        """Create instance of the LOTR API client.

        Method instantiates the `httpx.Client()` and sets: `base_url`,
        `headers` (Content-Type, Authorization), SSL verification is on and can not be changed

        Args:
            base_url : Base URL for the `httpx.CLient()`
            bearer_token :  Bearer auth token. Get here -https://the-one-api.dev/sign-up
        """
        settings = Settings(BEARER_TOKEN=bearer_token)

        headers = httpx.Headers(
            {
                "Content-Type": "application/json",
                "Authorization": f"Bearer {settings.BEARER_TOKEN}",
            },
        )
        self.http_client = httpx.Client(base_url=base_url, headers=headers, verify=True)

    def create_resource_class(self, resource_name: str) -> object:
        """Create an API resource (endpoint) class.

        Following endpoints are supported: `/movie`, `/movie/{id}`, `/movie/{id}/quote`, `/quote`,
        `/quote/{id}`

        Args:
            resource_name: Name of the resource (endpoint) class to create

        Returns:
            object: Resource class

        Raises:
            ValueError: If resource name is not supported
        """
        # We acknowledge that this is not the best way to do this, but to save time we use dirty way
        match resource_name.lower():
            case "movie":
                return Movie(self.http_client)
            case "quote":
                return Quote(self.http_client)
            case _:
                raise ValueError(f"Resource {resource_name} is not implemented")



import httpx

from mack_SDK.resources import models
from mack_SDK.tools.raise_for_status import raise_for_status


class Quote(object):
    """Quote resource class.

    This class is responsible for making requests to the /quote resource (endpoint)
    """

    http_client: httpx.Client

    def __init__(self, http_client: httpx.Client):
        """Initialize the class.

        Args:
            http_client : `httpx.Client` object
        """
        self.http_client = http_client

    def get_all_quotes(
        self,
        limit: int = 100,
        page: int = 1,
        offset: int = 0,
    ) -> models.QuotesModel:
        """Get all LOTR quotes.

        Args:
            limit : Number of quotes to return
            page : Page number to return
            offset : Offset

        Returns:
            models.QuotesModel : List of all LOTR quotes with pagination info

        Raises:
            HTTPStatusError : If status code is 4xx or 5xx # noqa: DAR402
        """
        params = {"limit": limit, "page": page, "offset": offset}  # noqa: WPS110
        res = self.http_client.request(method="GET", url="/quote", params=params)
        raise_for_status(res)
        return models.QuotesModel.parse_obj(res.json())

    def get_quote(self, item_id: str) -> models.QuoteModel:
        """Get a single quote by ID.

        Args:
            item_id : ID of the quote to get

        Returns:
            models.MovieModel : LOTR quote object

        Raises:
            HTTPStatusError : If status code is 4xx or 5xx # noqa: DAR402
        """
        res = self.http_client.request(method="GET", url=f"/quote/{item_id}")
        raise_for_status(res)
        return models.QuoteModel.parse_obj(res.json()["docs"][0])



import httpx

from mack_SDK.resources import models
from mack_SDK.tools.raise_for_status import raise_for_status


class Movie(object):
    """Movie resource class.

    This class is responsible for making requests to the /movie resource (endpoint)
    """

    http_client: httpx.Client

    def __init__(self, http_client: httpx.Client):
        """Initialize the class.

        Args:
            http_client : `httpx.Client` object
        """
        self.http_client = http_client

    def get_all_movies(
        self,
        limit: int = None,
        page: int = None,
        offset: int = None,
    ) -> models.MoviesModel:
        """Get all LOTR movies.

        Args:
            limit : Number of quotes to return
            page : Page number to return
            offset : Offset

        Returns:
            models.MoviesModel : List of all LOTR movies with pagination info

        Raises:
            HTTPStatusError : If status code is 4xx or 5xx # noqa: DAR402
        """
        params = {"limit": limit, "page": page, "offset": offset}  # noqa: WPS110
        res = self.http_client.request(method="GET", url="/movie", params=params)
        raise_for_status(res)
        return models.MoviesModel.parse_obj(res.json())

    def get_movie(self, item_id: str) -> models.MovieModel:
        """Get a single movie by ID.

        Args:
            item_id : ID of the movie to get

        Returns:
            models.MovieModel : LOTR movie object

        Raises:
            HTTPStatusError : If status code is 4xx or 5xx # noqa: DAR402
        """
        res = self.http_client.request(method="GET", url=f"/movie/{item_id}")
        raise_for_status(res)
        return models.MovieModel.parse_obj(res.json()["docs"][0])

    def get_quotes(
        self,
        item_id: str,
        limit: int = None,
        page: int = None,
        offset: int = None,
    ) -> models.QuotesModel:
        """Get quotes for a single movie.

        Args:
            item_id : ID of the movie to get quotes for
            limit : Number of quotes to return
            page : Page number to return
            offset : Offset

        Returns:
            models.QuotesModel : List of quotes for the movie with pagination info

        Raises:
            HTTPStatusError : If status code is 4xx or 5xx # noqa: DAR402
        """
        params = {"limit": limit, "page": page, "offset": offset}  # noqa: WPS110
        res = self.http_client.request(method="GET", url=f"/movie/{item_id}/quote", params=params)
        raise_for_status(res)
        return models.QuotesModel.parse_obj(res.json())
